/* Includes */
#include <stddef.h>
#include "stm32f10x.h"
#include "RFM70.h"

/*      Global variables----------------------------------------------------------*/
const u8 tx_buf[17] =
{ 0x30, 0x31, 0x32, 0x33, 0x34, 0x35, 0x36, 0x37, 0x38, 0x39, 0x3a, 0x3b, 0x3c,
		0x3d, 0x3e, 0x3f, 0x78 };
u8 rx_buf[MAX_PACKET_LEN];

#define MASTER	1
//#define SLAVE	1

//#define ZL31ARM	1
#define STM32VL_DISCOVERY	1

#define true 1
#define false 0

#ifdef ZL31ARM
	#define LED_PORT	GPIOB
	#define LED_PIN		GPIO_Pin_1

	void LED_on(){ GPIO_ResetBits(LED_PORT,LED_PIN); }
	void LED_off(){ GPIO_SetBits(LED_PORT,LED_PIN); }
#endif

#ifdef STM32VL_DISCOVERY
	#define LED_PORT	GPIOC
	#define LED_PIN		GPIO_Pin_9

	void LED_on(){ GPIO_SetBits(LED_PORT,LED_PIN); }
	void LED_off(){ GPIO_ResetBits(LED_PORT,LED_PIN); }
#endif

//	Initialization
void Initialization(void);
void CLK_Configuration(void);
void SPI_Configuration(void);
void init_mcu(void);
void init_port(void);

//	Delays
void power_on_delay(void);
void delay_200ms(void);
void delay_50ms(void);
void delay_5ms(void);
void delay_1ms(void);
void sub_program(void);

int main(void)
{
#ifdef MASTER
	u8 i, chksum;
	u8 temp_buf[32];
	int tempo = 0;
	u8 chktempo = false;
	Initialization();

	RFM70_SwitchToTxMode();

	while (1)
	{
		for (i = 0; i < 5; i++)
			delay_200ms(); // 1 sec delay
		for (i = 0; i < 17; i++)
		{
			temp_buf[i] = tx_buf[i]; // write data from tx_buf -> temp_buf, from where it will be sent
		}
		Send_Packet(W_TX_PAYLOAD_NOACK_CMD, temp_buf, 17);
		//      -------------------------------------------------
		RFM70_SwitchToRxMode();
		for (i = 0; i < tempo; i++)
			delay_1ms();
		Receive_Packet(rx_buf);

		if (rx_buf[0] != 0)
		{
			chktempo = true;
			chksum = 0;
			for (i = 0; i < 16; i++)
			{
				chksum += rx_buf[i];
			}

			if (chksum == rx_buf[16] && rx_buf[0] == 0x30) // 8 bit sum of the transmited packet [16]=0x3 78, [0]=0x30
			{
				LED_on();
				delay_50ms();
				delay_50ms();
				LED_off();
			}
		}
		if (chktempo == false)
			tempo++;
		RFM70_SwitchToTxMode();
	}
#endif

#ifdef SLAVE
	int i;
	u8 chksum, temp_buf[32];
	Initialization();
	RFM70_SwitchToRxMode();

	while (1)
	{
		Receive_Packet(rx_buf);

		if (rx_buf[0] != 0)
		{
			chksum = 0;
			for (i = 0; i < 16; i++)
			{
				chksum += rx_buf[i];
			}

			if (chksum == rx_buf[16] && rx_buf[0] == 0x30) // 8 bit sum of the transmited packet [16]=0x3 78, [0]=0x30
			{
				LED_on();
				delay_50ms();
				delay_50ms();
				LED_off();

				RFM70_SwitchToTxMode();
				//for(i = 0; i < 5; i++)delay_50ms();   // 1 sec delay
				for (i = 0; i < 17; i++)
				{
					temp_buf[i] = tx_buf[i]; // write data from tx_buf -> temp_buf, from where it will be sent
				}
				Send_Packet(W_TX_PAYLOAD_NOACK_CMD, temp_buf, 17);
			}
			RFM70_SwitchToRxMode();
		}
	}
#endif
}

void Initialization()
{
	CLK_Configuration();
	init_mcu();
	SPI_Configuration();
	power_on_delay(); 		// delay for RFM70
	RFM70_Initialize(); 	// RFM70 init - please look on RFM70.c
}

/*********************************************************
 Function: CLK_Configuration();

 Description:
 initialize mcu clock
 *********************************************************/
void CLK_Configuration(void)
{
	  //Konfiguracja RCC (doczenie do GPIOA oraz do SPI1)
	  RCC_APB2PeriphClockCmd(RCC_APB2Periph_GPIOA,ENABLE);
	  RCC_APB2PeriphClockCmd(RCC_APB2Periph_GPIOB,ENABLE);
	  RCC_APB2PeriphClockCmd(RCC_APB2Periph_SPI1,ENABLE);
//	  RCC_APB1PeriphClockCmd(RCC_APB1Periph_SPI2,ENABLE);
}

/*********************************************************
 Function: init_mcu();

 Description:
 initialize mcu.
 *********************************************************/
void init_mcu(void)
{
	init_port();
}
/*********************************************************
 Function: SPI_Configuration;

 Description:
 configure SPI.
 *********************************************************/
void SPI_Configuration(void)
{
	  //Struktury inicjalizacyjne
	  SPI_InitTypeDef SPI_InitStructure;

	  //Konfiguracja SPI1
	  SPI_InitStructure.SPI_Direction=SPI_Direction_2Lines_FullDuplex;
	  SPI_InitStructure.SPI_Mode=SPI_Mode_Master;
	  SPI_InitStructure.SPI_DataSize=SPI_DataSize_8b;
	  SPI_InitStructure.SPI_CPOL=SPI_CPOL_Low;
	  SPI_InitStructure.SPI_CPHA=SPI_CPHA_1Edge;
	  SPI_InitStructure.SPI_NSS=SPI_NSS_Soft;
	  SPI_InitStructure.SPI_BaudRatePrescaler=SPI_BaudRatePrescaler_256;
	  SPI_InitStructure.SPI_FirstBit=SPI_FirstBit_MSB;

	  SPI_Init(SPI1,&SPI_InitStructure);

	  SPI_Cmd(SPI1,ENABLE);
}



/*********************************************************
 Function: init_port();

 Description:
 initialize port.
 *********************************************************/

void init_port(void)
{
	  GPIO_InitTypeDef GPIO_InitStructure;

	  //PA4 - ChipSelect
	  GPIO_InitStructure.GPIO_Pin=CSN_PIN;
	  GPIO_InitStructure.GPIO_Mode=GPIO_Mode_Out_PP;
	  GPIO_InitStructure.GPIO_Speed=GPIO_Speed_50MHz;
	  GPIO_Init(CSN_PORT,&GPIO_InitStructure);

	  //PA4 - ChipEnable
	  GPIO_InitStructure.GPIO_Pin=CE_PIN;
	  GPIO_InitStructure.GPIO_Mode=GPIO_Mode_Out_PP;
	  GPIO_InitStructure.GPIO_Speed=GPIO_Speed_50MHz;
	  GPIO_Init(CE_PORT,&GPIO_InitStructure);

	  //PA5 - SCK
	  GPIO_InitStructure.GPIO_Pin=SCK_PIN;
	  GPIO_InitStructure.GPIO_Mode=GPIO_Mode_AF_PP;
	  GPIO_Init(SCK_PORT,&GPIO_InitStructure);

	  //PA7 - MOSI
	  GPIO_InitStructure.GPIO_Pin=MOSI_PIN;
	  GPIO_Init(MOSI_PORT,&GPIO_InitStructure);

	  //PA6 - MISO
	  GPIO_InitStructure.GPIO_Pin=MISO_PIN;
	  GPIO_InitStructure.GPIO_Mode=GPIO_Mode_IPU;
	  GPIO_InitStructure.GPIO_Speed=GPIO_Speed_50MHz;
	  GPIO_Init(MISO_PORT,&GPIO_InitStructure);

	  //LED
	  GPIO_InitStructure.GPIO_Pin=LED_PIN;
	  GPIO_InitStructure.GPIO_Mode=GPIO_Mode_Out_PP;
	  GPIO_InitStructure.GPIO_Speed=GPIO_Speed_2MHz;
	  GPIO_Init(LED_PORT,&GPIO_InitStructure);
	  LED_off();

	  //FIXME - czy przypadkiem nie korzysta z innych linii? (np. IRQ)
	/*
	GPIO_DeInit(GPIOA);
	GPIO_DeInit(GPIOG);
	GPIO_DeInit(GPIOD);
	//	RFM70 pins definition

	GPIO_Init(GPIOG, GPIO_PIN_0, GPIO_MODE_OUT_PP_LOW_FAST); // CSN chip select active low
	GPIO_Init(GPIOG, GPIO_PIN_1, GPIO_MODE_IN_PU_IT); // IRQ - Maskable interrupt pin active low
	GPIO_Init(GPIOA, GPIO_PIN_5, GPIO_MODE_OUT_PP_LOW_FAST); // CE (SPI master/slave)

	// 	Diodes on ports

	GPIO_Init(GPIOA, GPIO_PIN_4, GPIO_MODE_OUT_PP_LOW_FAST); // RED		- Send
	GPIO_Init(GPIOD, GPIO_PIN_0, GPIO_MODE_OUT_PP_LOW_FAST); // GREEN	- Receive

	//	Initialized values
	CE_Low();
	CSN_High();
	GPIO_WriteHigh(GPIOD, GPIO_PIN_0);	//ustawienia LEDw
	GPIO_WriteLow(GPIOA, GPIO_PIN_4);	//ustawienia LEDw
	*/
}

/*********************************************************
 Function:  interrupt ISR_timer()

 Description:

 *********************************************************/
/*
@far @interrupt void TIM4_UPD_OVF_IRQHandler(void) // Interrupt of TIM4 ovf
{
	unsigned char i;
	//;
	if (TIM4_GetFlagStatus(TIM4_IT_UPDATE) == SET)
	{

		TIM4_ClearFlag(TIM4_FLAG_UPDATE);
	}
}
*/
/*********************************************************
 Function:      power_on_delay()

 Description:
 Delay for RFM70
 *********************************************************/
void power_on_delay(void)
{
	unsigned int i;
	for (i = 0; i < 1000; i++)
	{
		delay_1ms();
	}
}
/********************************************************

 *********************************************************/
void delay_200ms(void)
{
	unsigned char j;
	for (j = 0; j < 40; j++)
	{
		delay_5ms();
	}
}
/*********************************************************
 Function: delay_50ms();

 Description:

 *********************************************************/
void delay_50ms(void)
{
	unsigned char j;
	for (j = 0; j < 10; j++)
	{
		delay_5ms();
	}
}
/*********************************************************
 Function: delay_5ms();

 Description:

 *********************************************************/
void delay_5ms(void)
{
	int i;
	for (i = 0; i < 5; i++) // 85*5
	{
		delay_1ms();
	}
}
/*********************************************************
 Function: delay_1ms();

 Description:

 *********************************************************/
void delay_1ms(void)
{
	int i;
#ifdef ZL31ARM
	for (i = 0; i < 2200; i++)
#endif
#ifdef STM32VL_DISCOVERY
		for (i = 0; i < 1000; i++)
#endif
	{
		__NOP();
	}
}

//**********************************BLACKGK********************************************/

#ifdef  USE_FULL_ASSERT

/**
 * @brief  Reports the name of the source file and the source line number
 *   where the assert_param error has occurred.
 * @param  file: pointer to the source file name
 * @param  line: assert_param error line source number
 * @retval None
 */
void assert_failed(uint8_t* file, uint32_t line)
{
	/* User can add his own implementation to report the file name and line number,
	 ex: printf("Wrong parameters value: file %s on line %d\r\n", file, line) */

	/* Infinite loop */
	while (1)
	{
	}
}
#endif

/*
 * Minimal __assert_func used by the assert() macro
 * */
void __assert_func(const char *file, int line, const char *func,
		const char *failedexpr)
{
	while (1)
	{
	}
}

/*
 * Minimal __assert() uses __assert__func()
 * */
void __assert(const char *file, int line, const char *failedexpr)
{
	__assert_func(file, line, NULL, failedexpr);
}

#ifdef USE_SEE
#ifndef USE_DEFAULT_TIMEOUT_CALLBACK
/**
 * @brief  Basic management of the timeout situation.
 * @param  None.
 * @retval sEE_FAIL.
 */
uint32_t sEE_TIMEOUT_UserCallback(void)
{
	/* Return with error code */
	return sEE_FAIL;
}
#endif
#endif /* USE_SEE */

